#include "mqtt_request.h"
#include "json/json.h"

namespace offcn
{
	/*
	{
		"path": "join",
		"rid" : "xxxid",
		"data" :
					{
						"pull_type" : "null"
					}
	}
	*/
	std::string MqttRequest::JoinRequest(std::string local_id)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "join";
		root["rid"] = local_id;

		Json::Value data;
		data["pull_type"] = "null";

		root["data"] = data;
		
		return Json::writeString(wBuilder, root);
	}
	/*
	{
		"path": "update",
		"rid" : "xxxid",
		"data" :
					{
						"pull_type" : "rtmp" or "null" or "kcp"
					}
	}
	*/
	std::string MqttRequest::UpdateRequest(std::string local_id, std::string pull_type)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "update";
		root["rid"] = local_id;

		Json::Value data;
		data["pull_type"] = pull_type;

		root["data"] = data;

		return Json::writeString(wBuilder, root);
	}
	/*
	{
		"path": "offer/answer",
		"rid" : "xxxid",
		"tid" : "remoteid",
		"candidate" : ""
	}
	*/
	std::string MqttRequest::Candidate(std::string local_id, std::string remote_id, std::string type, std::string candidate)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = type;
		root["rid"] = local_id;
		root["tid"] = remote_id;
		root["candidate"] = candidate;

		return Json::writeString(wBuilder, root);
	}

	static bool GetJsonValue(std::string &response, Json::Value &root)
	{
		Json::CharReaderBuilder readerBuilder;
		std::unique_ptr<Json::CharReader> const reader(readerBuilder.newCharReader());
		const char *start_pos = response.c_str();
		std::string err;
		if (!reader->parse(start_pos, start_pos + response.length(), &root, &err))
		{
			return false;
		}

		return true;
	}

	bool MqttRequest::ParseRequest(std::string &req, std::string &path, ReqValue &values)
	{
		Json::Value root;

		if (!GetJsonValue(req, root)) return false;

	}

	std::string MqttRequest::JoinResponse(std::string ids, std::string result)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "join";
		root["result"] = result;

		return Json::writeString(wBuilder, root);
	}
	std::string MqttRequest::UpdateResponse(std::string result)
	{
		Json::Value root;
		Json::StreamWriterBuilder wBuilder;

		root["path"] = "update";
		root["result"] = result;

		return Json::writeString(wBuilder, root);
	}
}